@@ -47,7 +47,7 @@ sdp_list_t *sdp_list_remove(sdp_list_t *list, void *d);
 sdp_list_t *sdp_list_insert_sorted(sdp_list_t *list, void *data, sdp_comp_func_t f);
 void        sdp_list_free(sdp_list_t *list, sdp_free_func_t f);
 
-static inline int sdp_list_len(const sdp_list_t *list) 
+static inline int sdp_list_len(const sdp_list_t *list)
 {
 	int n = 0;
 	for (; list; list = list->next)
@@ -90,7 +90,7 @@ typedef struct {
 	int state;
 	int local;
 	int flags;
-	uint16_t tid;	// Current transaction ID
+	uint16_t tid;	/* Current transaction ID */
 	void *priv;
 } sdp_session_t;
 
@@ -118,9 +118,9 @@ typedef void sdp_callback_t(uint8_t type, uint16_t status, uint8_t *rsp, size_t
 
 /*
  * create an L2CAP connection to a Bluetooth device
- * 
+ *
  * INPUT:
- *  
+ *
  *  bdaddr_t *src:
  *	Address of the local device to use to make the connection
  *	(or BDADDR_ANY)
@@ -174,29 +174,29 @@ int sdp_set_uuidseq_attr(sdp_record_t *rec, uint16_t attr, sdp_list_t *seq);
 int sdp_get_uuidseq_attr(const sdp_record_t *rec, uint16_t attr, sdp_list_t **seqp);
 
 /*
- * NOTE that none of the functions below will update the SDP server, 
+ * NOTE that none of the functions below will update the SDP server,
  * unless the {register, update}sdp_record_t() function is invoked.
- * All functions which return an integer value, return 0 on success 
+ * All functions which return an integer value, return 0 on success
  * or -1 on failure.
  */
 
 /*
  * Create an attribute and add it to the service record's attribute list.
- * This consists of the data type descriptor of the attribute, 
+ * This consists of the data type descriptor of the attribute,
  * the value of the attribute and the attribute identifier.
  */
 int sdp_attr_add_new(sdp_record_t *rec, uint16_t attr, uint8_t dtd, const void *p);
 
 /*
  * Set the information attributes of the service record.
- * The set of attributes comprises service name, description 
+ * The set of attributes comprises service name, description
  * and provider name
  */
 void sdp_set_info_attr(sdp_record_t *rec, const char *name, const char *prov, const char *desc);
 
 /*
  * Set the ServiceClassID attribute to the sequence specified by seq.
- * Note that the identifiers need to be in sorted order from the most 
+ * Note that the identifiers need to be in sorted order from the most
  * specific to the most generic service class that this service
  * conforms to.
  */
@@ -207,8 +207,8 @@ static inline int sdp_set_service_classes(sdp_record_t *rec, sdp_list_t *seq)
 
 /*
  * Get the service classes to which the service conforms.
- * 
- * When set, the list contains elements of ServiceClassIdentifer(uint16_t) 
+ *
+ * When set, the list contains elements of ServiceClassIdentifer(uint16_t)
  * ordered from most specific to most generic
  */
 static inline int sdp_get_service_classes(const sdp_record_t *rec, sdp_list_t **seqp)
@@ -218,8 +218,8 @@ static inline int sdp_get_service_classes(const sdp_record_t *rec, sdp_list_t **
 
 /*
  * Set the BrowseGroupList attribute to the list specified by seq.
- * 
- * A service can belong to one or more service groups 
+ *
+ * A service can belong to one or more service groups
  * and the list comprises such group identifiers (UUIDs)
  */
 static inline int sdp_set_browse_groups(sdp_record_t *rec, sdp_list_t *seq)
@@ -238,24 +238,24 @@ int sdp_set_access_protos(sdp_record_t *rec, const sdp_list_t *proto);
 int sdp_set_add_access_protos(sdp_record_t *rec, const sdp_list_t *proto);
 
 /*
- * Get protocol port (i.e. PSM for L2CAP, Channel for RFCOMM) 
+ * Get protocol port (i.e. PSM for L2CAP, Channel for RFCOMM)
  */
 int sdp_get_proto_port(const sdp_list_t *list, int proto);
 
 /*
- * Get protocol descriptor. 
+ * Get protocol descriptor.
  */
 sdp_data_t *sdp_get_proto_desc(sdp_list_t *list, int proto);
 
 /*
- * Set the LanguageBase attributes to the values specified in list 
- * (a linked list of sdp_lang_attr_t objects, one for each language in 
+ * Set the LanguageBase attributes to the values specified in list
+ * (a linked list of sdp_lang_attr_t objects, one for each language in
  * which user-visible attributes are present).
  */
 int sdp_set_lang_attr(sdp_record_t *rec, const sdp_list_t *list);
 
 /*
- * Set the ServiceInfoTimeToLive attribute of the service.  
+ * Set the ServiceInfoTimeToLive attribute of the service.
  * This is the number of seconds that this record is guaranteed
  * not to change after being obtained by a client.
  */
@@ -266,8 +266,8 @@ static inline int sdp_set_service_ttl(sdp_record_t *rec, uint32_t ttl)
 
 /*
  * Set the ServiceRecordState attribute of a service. This is
- * guaranteed to change if there is any kind of modification to 
- * the record. 
+ * guaranteed to change if there is any kind of modification to
+ * the record.
  */
 static inline int sdp_set_record_state(sdp_record_t *rec, uint32_t state)
 {
@@ -275,7 +275,7 @@ static inline int sdp_set_record_state(sdp_record_t *rec, uint32_t state)
 }
 
 /*
- * Set the ServiceID attribute of a service. 
+ * Set the ServiceID attribute of a service.
  */
 void sdp_set_service_id(sdp_record_t *rec, uuid_t uuid);
 
@@ -286,7 +286,7 @@ void sdp_set_group_id(sdp_record_t *rec, uuid_t grouuuid);
 
 /*
  * Set the ServiceAvailability attribute of a service.
- * 
+ *
  * Note that this represents the relative availability
  * of the service: 0x00 means completely unavailable;
  * 0xFF means maximum availability.
@@ -298,7 +298,7 @@ static inline int sdp_set_service_avail(sdp_record_t *rec, uint8_t avail)
 
 /*
  * Set the profile descriptor list attribute of a record.
- * 
+ *
  * Each element in the list is an object of type
  * sdp_profile_desc_t which is a definition of the
  * Bluetooth profile that this service conforms to.
@@ -307,41 +307,41 @@ int sdp_set_profile_descs(sdp_record_t *rec, const sdp_list_t *desc);
 
 /*
  * Set URL attributes of a record.
- * 
- * ClientExecutableURL: a URL to a client's platform specific (WinCE, 
+ *
+ * ClientExecutableURL: a URL to a client's platform specific (WinCE,
  * PalmOS) executable code that can be used to access this service.
- * 
+ *
  * DocumentationURL: a URL pointing to service documentation
- * 
+ *
  * IconURL: a URL to an icon that can be used to represent this service.
- * 
+ *
  * Note: pass NULL for any URLs that you don't want to set or remove
  */
 void sdp_set_url_attr(sdp_record_t *rec, const char *clientExecURL, const char *docURL, const char *iconURL);
 
 /*
- * a service search request. 
- * 
+ * a service search request.
+ *
  *  INPUT :
- * 
+ *
  *    sdp_list_t *search
  *      list containing elements of the search
  *      pattern. Each entry in the list is a UUID
  *      of the service to be searched
- * 
+ *
  *    uint16_t max_rec_num
  *       An integer specifying the maximum number of
  *       entries that the client can handle in the response.
- * 
+ *
  *  OUTPUT :
- * 
+ *
  *    int return value
- *      0 
+ *      0
  *        The request completed successfully. This does not
  *        mean the requested services were found
  *      -1
  *        The request completed unsuccessfully
- * 
+ *
  *    sdp_list_t *rsp_list
  *      This variable is set on a successful return if there are
  *      non-zero service handles. It is a singly linked list of
@@ -350,33 +350,33 @@ void sdp_set_url_attr(sdp_record_t *rec, const char *clientExecURL, const char *
 int sdp_service_search_req(sdp_session_t *session, const sdp_list_t *search, uint16_t max_rec_num, sdp_list_t **rsp_list);
 
 /*
- *  a service attribute request. 
- * 
+ *  a service attribute request.
+ *
  *  INPUT :
- * 
+ *
  *    uint32_t handle
  *      The handle of the service for which the attribute(s) are
  *      requested
- * 
+ *
  *    sdp_attrreq_type_t reqtype
  *      Attribute identifiers are 16 bit unsigned integers specified
  *      in one of 2 ways described below :
  *      SDP_ATTR_REQ_INDIVIDUAL - 16bit individual identifiers
  *         They are the actual attribute identifiers in ascending order
- * 
+ *
  *      SDP_ATTR_REQ_RANGE - 32bit identifier range
  *         The high-order 16bits is the start of range
  *         the low-order 16bits are the end of range
  *         0x0000 to 0xFFFF gets all attributes
- * 
+ *
  *    sdp_list_t *attrid_list
  *      Singly linked list containing attribute identifiers desired.
- *      Every element is either a uint16_t(attrSpec = SDP_ATTR_REQ_INDIVIDUAL)  
+ *      Every element is either a uint16_t(attrSpec = SDP_ATTR_REQ_INDIVIDUAL)
  *      or a uint32_t(attrSpec=SDP_ATTR_REQ_RANGE)
- * 
+ *
  *  OUTPUT :
  *    int return value
- *      0 
+ *      0
  *        The request completed successfully. This does not
  *        mean the requested services were found
  *      -1
@@ -388,38 +388,38 @@ sdp_record_t *sdp_service_attr_req(sdp_session_t *session, uint32_t handle, sdp_
  *  This is a service search request combined with the service
  *  attribute request. First a service class match is done and
  *  for matching service, requested attributes are extracted
- * 
+ *
  *  INPUT :
- * 
+ *
  *    sdp_list_t *search
  *      Singly linked list containing elements of the search
  *      pattern. Each entry in the list is a UUID(DataTypeSDP_UUID16)
  *      of the service to be searched
- * 
+ *
  *    AttributeSpecification attrSpec
  *      Attribute identifiers are 16 bit unsigned integers specified
  *      in one of 2 ways described below :
  *      SDP_ATTR_REQ_INDIVIDUAL - 16bit individual identifiers
  *         They are the actual attribute identifiers in ascending order
- * 
+ *
  *      SDP_ATTR_REQ_RANGE - 32bit identifier range
  *         The high-order 16bits is the start of range
  *         the low-order 16bits are the end of range
  *         0x0000 to 0xFFFF gets all attributes
- * 
+ *
  *    sdp_list_t *attrid_list
  *      Singly linked list containing attribute identifiers desired.
- *      Every element is either a uint16_t(attrSpec = SDP_ATTR_REQ_INDIVIDUAL)  
+ *      Every element is either a uint16_t(attrSpec = SDP_ATTR_REQ_INDIVIDUAL)
  *      or a uint32_t(attrSpec=SDP_ATTR_REQ_RANGE)
- * 
+ *
  *  OUTPUT :
  *    int return value
- *      0 
+ *      0
  *        The request completed successfully. This does not
  *        mean the requested services were found
  *      -1
  *        The request completed unsuccessfully due to a timeout
- * 
+ *
  *    sdp_list_t *rsp_list
  *      This variable is set on a successful return to point to
  *      service(s) found. Each element of this list is of type
@@ -434,12 +434,12 @@ sdp_record_t *sdp_record_alloc(void);
 void sdp_record_free(sdp_record_t *rec);
 
 /*
- * Register a service record. 
- * 
- * Note: It is the responsbility of the Service Provider to create the 
+ * Register a service record.
+ *
+ * Note: It is the responsbility of the Service Provider to create the
  * record first and set its attributes using setXXX() methods.
- * 
- * The service provider must then call sdp_record_register() to make 
+ *
+ * The service provider must then call sdp_record_register() to make
  * the service record visible to SDP clients.  This function returns 0
  * on success or -1 on failure (and sets errno).
  */
@@ -474,9 +474,9 @@ uuid_t *sdp_uuid128_create(uuid_t *uuid, const void *data);
 int sdp_uuid16_cmp(const void *p1, const void *p2);
 int sdp_uuid128_cmp(const void *p1, const void *p2);
 int sdp_uuid_cmp(const void *p1, const void *p2);
-uuid_t *sdp_uuid_to_uuid128(uuid_t *uuid);
-void sdp_uuid16_to_uuid128(uuid_t *uuid128, uuid_t *uuid16);
-void sdp_uuid32_to_uuid128(uuid_t *uuid128, uuid_t *uuid32);
+uuid_t *sdp_uuid_to_uuid128(const uuid_t *uuid);
+void sdp_uuid16_to_uuid128(uuid_t *uuid128, const uuid_t *uuid16);
+void sdp_uuid32_to_uuid128(uuid_t *uuid128, const uuid_t *uuid32);
 int sdp_uuid128_to_uuid(uuid_t *uuid);
 int sdp_uuid_to_proto(uuid_t *uuid);
 int sdp_uuid_extract(const uint8_t *buffer, int bufsize, uuid_t *uuid, int *scanned);
@@ -493,14 +493,14 @@ int sdp_svclass_uuid2strn(const uuid_t *uuid, char *str, size_t n);
 int sdp_profile_uuid2strn(const uuid_t *uuid, char *str, size_t n);
 
 /*
- * In all the sdp_get_XXX(handle, XXX *xxx) functions below, 
+ * In all the sdp_get_XXX(handle, XXX *xxx) functions below,
  * the XXX * is set to point to the value, should it exist
  * and 0 is returned. If the value does not exist, -1 is
  * returned and errno set to ENODATA.
  *
  * In all the methods below, the memory management rules are
  * simple. Don't free anything! The pointer returned, in the
- * case of constructed types, is a pointer to the contents 
+ * case of constructed types, is a pointer to the contents
  * of the sdp_record_t.
  */
 
@@ -516,7 +516,7 @@ int sdp_get_add_access_protos(const sdp_record_t *rec, sdp_list_t **protos);
 
 /*
  * Extract the list of browse groups to which the service belongs.
- * When set, seqp contains elements of GroupID (uint16_t) 
+ * When set, seqp contains elements of GroupID (uint16_t)
  */
 static inline int sdp_get_browse_groups(const sdp_record_t *rec, sdp_list_t **seqp)
 {
@@ -524,9 +524,9 @@ static inline int sdp_get_browse_groups(const sdp_record_t *rec, sdp_list_t **se
 }
 
 /*
- * Extract language attribute meta-data of the service record. 
+ * Extract language attribute meta-data of the service record.
  * For each language in the service record, LangSeq has a struct of type
- * sdp_lang_attr_t. 
+ * sdp_lang_attr_t.
  */
 int sdp_get_lang_attr(const sdp_record_t *rec, sdp_list_t **langSeq);
 
@@ -540,8 +540,8 @@ int sdp_get_lang_attr(const sdp_record_t *rec, sdp_list_t **langSeq);
 int sdp_get_profile_descs(const sdp_record_t *rec, sdp_list_t **profDesc);
 
 /*
- * Extract SDP server version numbers 
- * 
+ * Extract SDP server version numbers
+ *
  * Note: that this is an attribute of the SDP server only and
  * contains a list of uint16_t each of which represent the
  * major and minor SDP version numbers supported by this server
@@ -624,6 +624,8 @@ void sdp_pattern_add_uuidseq(sdp_record_t *rec, sdp_list_t *seq);
 
 int sdp_send_req_w4_rsp(sdp_session_t *session, uint8_t *req, uint8_t *rsp, uint32_t reqsize, uint32_t *rspsize);
 
+void sdp_add_lang_attr(sdp_record_t *rec);
+
 #ifdef __cplusplus
 }
 #endif
