@@ -37,10 +37,8 @@
 #include <dirent.h>
 
 #include <bluetooth/bluetooth.h>
-#include <bluetooth/uuid.h>
 #include <bluetooth/sdp.h>
 #include <bluetooth/sdp_lib.h>
-#include <bluetooth/mgmt.h>
 
 #include <glib.h>
 #include <dbus/dbus.h>
@@ -49,6 +47,8 @@
 
 #include "log.h"
 
+#include "lib/uuid.h"
+#include "lib/mgmt.h"
 #include "attrib/att.h"
 #include "hcid.h"
 #include "adapter.h"
@@ -139,7 +139,7 @@ struct att_callbacks {
 struct btd_device {
 	bdaddr_t	bdaddr;
 	uint8_t		bdaddr_type;
-	gchar		*path;
+	char		*path;
 	bool		svc_resolved;
 	GSList		*eir_uuids;
 	char		name[MAX_NAME_LENGTH + 1];
@@ -150,6 +150,7 @@ struct btd_device {
 	uint16_t	product;
 	uint16_t	version;
 	uint16_t	appearance;
+	char		*modalias;
 	struct btd_adapter	*adapter;
 	GSList		*uuids;
 	GSList		*primaries;		/* List of primary services */
@@ -213,7 +214,7 @@ static gboolean store_device_info_cb(gpointer user_data)
 	char device_addr[18];
 	char *str;
 	char class[9];
-	gchar **uuids = NULL;
+	char **uuids = NULL;
 	gsize length = 0;
 
 	device->store_id = 0;
@@ -288,11 +289,11 @@ static gboolean store_device_info_cb(gpointer user_data)
 		GSList *l;
 		int i;
 
-		uuids = g_new0(gchar *, g_slist_length(device->uuids) + 1);
+		uuids = g_new0(char *, g_slist_length(device->uuids) + 1);
 		for (i = 0, l = device->uuids; l; l = g_slist_next(l), i++)
 			uuids[i] = l->data;
 		g_key_file_set_string_list(key_file, "General", "Services",
-						(const gchar **)uuids, i);
+						(const char **)uuids, i);
 	} else {
 		g_key_file_remove_key(key_file, "General", "Services", NULL);
 	}
@@ -422,6 +423,7 @@ static void device_free(gpointer user_data)
 
 	g_free(device->path);
 	g_free(device->alias);
+	g_free(device->modalias);
 	g_free(device);
 }
 
@@ -513,8 +515,10 @@ static void set_alias(GDBusPendingPropertySet id, const char *alias,
 
 	/* No change */
 	if ((device->alias == NULL && g_str_equal(alias, "")) ||
-					g_strcmp0(device->alias, alias) == 0)
-		return g_dbus_pending_property_success(id);
+					g_strcmp0(device->alias, alias) == 0) {
+		g_dbus_pending_property_success(id);
+		return;
+	}
 
 	g_free(device->alias);
 	device->alias = g_str_equal(alias, "") ? NULL : g_strdup(alias);
@@ -533,10 +537,12 @@ static void dev_property_set_alias(const GDBusPropertyTable *property,
 {
 	const char *alias;
 
-	if (dbus_message_iter_get_arg_type(value) != DBUS_TYPE_STRING)
-		return g_dbus_pending_property_error(id,
+	if (dbus_message_iter_get_arg_type(value) != DBUS_TYPE_STRING) {
+		g_dbus_pending_property_error(id,
 					ERROR_INTERFACE ".InvalidArguments",
 					"Invalid arguments in method call");
+		return;
+	}
 
 	dbus_message_iter_get_basic(value, &alias);
 
@@ -635,121 +641,6 @@ static gboolean dev_property_get_icon(const GDBusPropertyTable *property,
 	return TRUE;
 }
 
-static gboolean dev_property_exists_vendor(const GDBusPropertyTable *property,
-								void *data)
-{
-	struct btd_device *device = data;
-
-	if (device->vendor_src < 1 || device->vendor_src > 2)
-		return FALSE;
-
-	return !!device->vendor;
-}
-
-static gboolean dev_property_get_vendor(const GDBusPropertyTable *property,
-					DBusMessageIter *iter, void *data)
-{
-	struct btd_device *device = data;
-
-	if (device->vendor_src < 1 || device->vendor_src > 2)
-		return FALSE;
-
-	if (!device->vendor)
-		return FALSE;
-
-	dbus_message_iter_append_basic(iter, DBUS_TYPE_UINT16,
-							&device->vendor);
-	return TRUE;
-}
-
-static gboolean dev_property_exists_vendor_src(
-				const GDBusPropertyTable *property, void *data)
-{
-	struct btd_device *device = data;
-
-	if (device->vendor_src < 1 || device->vendor_src > 2)
-		return FALSE;
-
-	return TRUE;
-}
-
-static gboolean dev_property_get_vendor_src(const GDBusPropertyTable *property,
-					DBusMessageIter *iter, void *data)
-{
-	struct btd_device *device = data;
-	const char *str;
-
-	switch (device->vendor_src) {
-	case 1:
-		str = "bluetooth";
-		break;
-	case 2:
-		str = "usb";
-		break;
-	default:
-		return FALSE;
-	}
-
-	dbus_message_iter_append_basic(iter, DBUS_TYPE_STRING, &str);
-
-	return TRUE;
-}
-
-static gboolean dev_property_exists_product(const GDBusPropertyTable *property,
-								void *data)
-{
-	struct btd_device *device = data;
-
-	if (device->vendor_src < 1 || device->vendor_src > 2)
-		return FALSE;
-
-	return !!device->vendor;
-}
-
-static gboolean dev_property_get_product(const GDBusPropertyTable *property,
-					DBusMessageIter *iter, void *data)
-{
-	struct btd_device *device = data;
-
-	if (device->vendor_src < 1 || device->vendor_src > 2)
-		return FALSE;
-
-	if (!device->vendor)
-		return FALSE;
-
-	dbus_message_iter_append_basic(iter, DBUS_TYPE_UINT16,
-							&device->product);
-
-	return TRUE;
-}
-
-static gboolean dev_property_exists_version(const GDBusPropertyTable *property,
-								void *data)
-{
-	struct btd_device *device = data;
-
-	if (device->vendor_src < 1 || device->vendor_src > 2)
-		return FALSE;
-
-	return !!device->vendor;
-}
-
-static gboolean dev_property_get_version(const GDBusPropertyTable *property,
-					DBusMessageIter *iter, void *data)
-{
-	struct btd_device *device = data;
-
-	if (device->vendor_src < 1 || device->vendor_src > 2)
-		return FALSE;
-
-	if (!device->vendor)
-		return FALSE;
-
-	dbus_message_iter_append_basic(iter, DBUS_TYPE_UINT16,
-							&device->version);
-	return TRUE;
-}
-
 static gboolean dev_property_get_paired(const GDBusPropertyTable *property,
 					DBusMessageIter *iter, void *data)
 {
@@ -806,8 +697,10 @@ static void set_trust(GDBusPendingPropertySet id, gboolean value, void *data)
 {
 	struct btd_device *device = data;
 
-	if (device->trusted == value)
-		return g_dbus_pending_property_success(id);
+	if (device->trusted == value) {
+		g_dbus_pending_property_success(id);
+		return;
+	}
 
 	device->trusted = value;
 
@@ -825,10 +718,12 @@ static void dev_property_set_trusted(const GDBusPropertyTable *property,
 {
 	dbus_bool_t b;
 
-	if (dbus_message_iter_get_arg_type(value) != DBUS_TYPE_BOOLEAN)
-		return g_dbus_pending_property_error(id,
+	if (dbus_message_iter_get_arg_type(value) != DBUS_TYPE_BOOLEAN) {
+		g_dbus_pending_property_error(id,
 					ERROR_INTERFACE ".InvalidArguments",
 					"Invalid arguments in method call");
+		return;
+	}
 
 	dbus_message_iter_get_basic(value, &b);
 
@@ -878,10 +773,12 @@ static void dev_property_set_blocked(const GDBusPropertyTable *property,
 {
 	dbus_bool_t b;
 
-	if (dbus_message_iter_get_arg_type(value) != DBUS_TYPE_BOOLEAN)
-		return g_dbus_pending_property_error(id,
+	if (dbus_message_iter_get_arg_type(value) != DBUS_TYPE_BOOLEAN) {
+		g_dbus_pending_property_error(id,
 					ERROR_INTERFACE ".InvalidArguments",
 					"Invalid arguments in method call");
+		return;
+	}
 
 	dbus_message_iter_get_basic(value, &b);
 
@@ -923,15 +820,26 @@ static gboolean dev_property_get_uuids(const GDBusPropertyTable *property,
 	return TRUE;
 }
 
-static gboolean dev_property_exists_uuids(const GDBusPropertyTable *property,
+static gboolean dev_property_get_modalias(const GDBusPropertyTable *property,
+					DBusMessageIter *iter, void *data)
+{
+	struct btd_device *device = data;
+
+	if (!device->modalias)
+		return FALSE;
+
+	dbus_message_iter_append_basic(iter, DBUS_TYPE_STRING,
+							&device->modalias);
+
+	return TRUE;
+}
+
+static gboolean dev_property_exists_modalias(const GDBusPropertyTable *property,
 								void *data)
 {
-	struct btd_device *dev = data;
+	struct btd_device *device = data;
 
-	if (dev->svc_resolved)
-		return dev->uuids ? TRUE : FALSE;
-	else
-		return dev->eir_uuids ? TRUE : FALSE;
+	return device->modalias ? TRUE : FALSE;
 }
 
 static gboolean dev_property_get_adapter(const GDBusPropertyTable *property,
@@ -1146,13 +1054,13 @@ void device_profile_connected(struct btd_device *dev,
 
 	dev->pending = g_slist_remove(dev->pending, profile);
 
-	if (!err) {
+	if (!err)
 		dev->connected_profiles =
 				g_slist_append(dev->connected_profiles,
 								profile);
-		if (connect_next(dev) == 0)
-			return;
-	}
+
+	if (connect_next(dev) == 0)
+		return;
 
 	if (!dev->connect)
 		return;
@@ -1629,22 +1537,15 @@ static const GDBusPropertyTable device_properties[] = {
 					dev_property_exists_appearance },
 	{ "Icon", "s", dev_property_get_icon, NULL,
 					dev_property_exists_icon },
-	{ "Vendor", "q", dev_property_get_vendor, NULL,
-					dev_property_exists_vendor },
-	{ "VendorSource", "s", dev_property_get_vendor_src, NULL,
-					dev_property_exists_vendor_src },
-	{ "Product", "q", dev_property_get_product, NULL,
-					dev_property_exists_product },
-	{ "Version", "q", dev_property_get_version, NULL,
-					dev_property_exists_version },
 	{ "Paired", "b", dev_property_get_paired },
 	{ "Trusted", "b", dev_property_get_trusted, dev_property_set_trusted },
 	{ "Blocked", "b", dev_property_get_blocked, dev_property_set_blocked },
 	{ "LegacyPairing", "b", dev_property_get_legacy },
 	{ "RSSI", "n", dev_property_get_rssi, NULL, dev_property_exists_rssi },
 	{ "Connected", "b", dev_property_get_connected },
-	{ "UUIDs", "as", dev_property_get_uuids, NULL,
-						dev_property_exists_uuids },
+	{ "UUIDs", "as", dev_property_get_uuids },
+	{ "Modalias", "s", dev_property_get_modalias, NULL,
+						dev_property_exists_modalias },
 	{ "Adapter", "o", dev_property_get_adapter },
 	{ }
 };
@@ -1740,52 +1641,8 @@ void device_remove_disconnect_watch(struct btd_device *device, guint id)
 	}
 }
 
-static void device_set_vendor(struct btd_device *device, uint16_t value)
-{
-	if (device->vendor == value)
-		return;
-
-	device->vendor = value;
-
-	g_dbus_emit_property_changed(btd_get_dbus_connection(), device->path,
-						DEVICE_INTERFACE, "Vendor");
-}
-
-static void device_set_vendor_src(struct btd_device *device, uint16_t value)
-{
-	if (device->vendor_src == value)
-		return;
-
-	device->vendor_src = value;
-
-	g_dbus_emit_property_changed(btd_get_dbus_connection(), device->path,
-					DEVICE_INTERFACE, "VendorSource");
-}
-
-static void device_set_product(struct btd_device *device, uint16_t value)
-{
-	if (device->product == value)
-		return;
-
-	device->product = value;
-
-	g_dbus_emit_property_changed(btd_get_dbus_connection(), device->path,
-						DEVICE_INTERFACE, "Product");
-}
-
-static void device_set_version(struct btd_device *device, uint16_t value)
-{
-	if (device->version == value)
-		return;
-
-	device->version = value;
-
-	g_dbus_emit_property_changed(btd_get_dbus_connection(), device->path,
-						DEVICE_INTERFACE, "Version");
-}
-
 static char *load_cached_name(struct btd_device *device, const char *local,
-				const gchar *peer)
+				const char *peer)
 {
 	char filename[PATH_MAX + 1];
 	GKeyFile *key_file;
@@ -1813,13 +1670,13 @@ failed:
 	return str;
 }
 
-static void load_info(struct btd_device *device, const gchar *local,
-			const gchar *peer, GKeyFile *key_file)
+static void load_info(struct btd_device *device, const char *local,
+			const char *peer, GKeyFile *key_file)
 {
 	char *str;
 	gboolean store_needed = FALSE;
 	gboolean blocked;
-	gchar **uuids;
+	char **uuids;
 	int source, vendor, product, version;
 	char **techno, **t;
 	gboolean bredr = FALSE;
@@ -1910,7 +1767,7 @@ next:
 	uuids = g_key_file_get_string_list(key_file, "General", "Services",
 						NULL, NULL);
 	if (uuids) {
-		gchar **uuid;
+		char **uuid;
 
 		for (uuid = uuids; *uuid; uuid++) {
 			GSList *match;
@@ -1930,27 +1787,24 @@ next:
 	/* Load device id */
 	source = g_key_file_get_integer(key_file, "DeviceID", "Source", NULL);
 	if (source) {
-		device_set_vendor_src(device, source);
-
 		vendor = g_key_file_get_integer(key_file, "DeviceID",
 							"Vendor", NULL);
-		device_set_vendor(device, vendor);
 
 		product = g_key_file_get_integer(key_file, "DeviceID",
 							"Product", NULL);
-		device_set_product(device, product);
 
 		version = g_key_file_get_integer(key_file, "DeviceID",
 							"Version", NULL);
-		device_set_version(device, version);
+
+		btd_device_set_pnpid(device, source, vendor, product, version);
 	}
 
 	if (store_needed)
 		store_device_info(device);
 }
 
-static void load_att_info(struct btd_device *device, const gchar *local,
-				const gchar *peer)
+static void load_att_info(struct btd_device *device, const char *local,
+				const char *peer)
 {
 	char filename[PATH_MAX + 1];
 	GKeyFile *key_file;
@@ -2039,11 +1893,11 @@ static void load_att_info(struct btd_device *device, const gchar *local,
 }
 
 static struct btd_device *device_new(struct btd_adapter *adapter,
-				const gchar *address)
+				const char *address)
 {
-	gchar *address_up;
+	char *address_up;
 	struct btd_device *device;
-	const gchar *adapter_path = adapter_get_path(adapter);
+	const char *adapter_path = adapter_get_path(adapter);
 
 	device = g_try_malloc0(sizeof(struct btd_device));
 	if (device == NULL)
@@ -2092,7 +1946,7 @@ struct btd_device *device_create_from_storage(struct btd_adapter *adapter,
 }
 
 struct btd_device *device_create(struct btd_adapter *adapter,
-				const gchar *address, uint8_t bdaddr_type)
+				const char *address, uint8_t bdaddr_type)
 {
 	struct btd_device *device;
 	const bdaddr_t *src;
@@ -2320,7 +2174,7 @@ void device_remove(struct btd_device *device, gboolean remove_stored)
 	btd_device_unref(device);
 }
 
-gint device_address_cmp(struct btd_device *device, const gchar *address)
+gint device_address_cmp(struct btd_device *device, const char *address)
 {
 	char addr[18];
 
@@ -2645,7 +2499,7 @@ static void update_bredr_services(struct browse_req *req, sdp_list_t *recs)
 	for (seq = recs; seq; seq = seq->next) {
 		sdp_record_t *rec = (sdp_record_t *) seq->data;
 		sdp_list_t *svcclass = NULL;
-		gchar *profile_uuid;
+		char *profile_uuid;
 		GSList *l;
 
 		if (!rec)
@@ -3478,7 +3332,7 @@ const bdaddr_t *device_get_address(struct btd_device *device)
 	return &device->bdaddr;
 }
 
-const gchar *device_get_path(struct btd_device *device)
+const char *device_get_path(struct btd_device *device)
 {
 	if (!device)
 		return NULL;
@@ -3624,8 +3478,7 @@ void device_bonding_complete(struct btd_device *device, uint8_t status)
 
 	DBG("bonding %p status 0x%02x", bonding, status);
 
-	if (auth && (auth->type == AUTH_TYPE_NOTIFY_PASSKEY
-		     || auth->type == AUTH_TYPE_NOTIFY_PINCODE) && auth->agent)
+	if (auth && auth->agent)
 		agent_cancel(auth->agent);
 
 	if (status) {
@@ -4115,7 +3968,7 @@ struct btd_device *btd_device_ref(struct btd_device *device)
 
 void btd_device_unref(struct btd_device *device)
 {
-	gchar *path;
+	char *path;
 
 	device->ref--;
 
@@ -4241,14 +4094,19 @@ gboolean btd_device_remove_attio_callback(struct btd_device *device, guint id)
 	return TRUE;
 }
 
-void btd_device_set_pnpid(struct btd_device *device, uint8_t vendor_id_src,
-			uint16_t vendor_id, uint16_t product_id,
-			uint16_t product_ver)
+void btd_device_set_pnpid(struct btd_device *device, uint16_t source,
+			uint16_t vendor, uint16_t product, uint16_t version)
 {
-	device_set_vendor(device, vendor_id);
-	device_set_vendor_src(device, vendor_id_src);
-	device_set_product(device, product_id);
-	device_set_version(device, product_ver);
+	device->vendor_src = source;
+	device->vendor = vendor;
+	device->product = product;
+	device->version = version;
+
+	g_free(device->modalias);
+	device->modalias = bt_modalias(source, vendor, product, version);
+
+	g_dbus_emit_property_changed(btd_get_dbus_connection(), device->path,
+						DEVICE_INTERFACE, "Modalias");
 
 	store_device_info(device);
 }
