@@ -42,7 +42,7 @@ uint16_t btd_device_get_vendor_src(struct btd_device *device);
 uint16_t btd_device_get_product(struct btd_device *device);
 uint16_t btd_device_get_version(struct btd_device *device);
 void device_remove(struct btd_device *device, gboolean remove_stored);
-gint device_address_cmp(struct btd_device *device, const gchar *address);
+gint device_address_cmp(struct btd_device *device, const char *address);
 gint device_bdaddr_cmp(struct btd_device *device, bdaddr_t *bdaddr);
 GSList *device_get_uuids(struct btd_device *device);
 void device_probe_profiles(struct btd_device *device, GSList *profiles);
@@ -59,7 +59,7 @@ void device_probe_profile(gpointer a, gpointer b);
 void device_remove_profile(gpointer a, gpointer b);
 struct btd_adapter *device_get_adapter(struct btd_device *device);
 const bdaddr_t *device_get_address(struct btd_device *device);
-const gchar *device_get_path(struct btd_device *device);
+const char *device_get_path(struct btd_device *device);
 gboolean device_is_bredr(struct btd_device *device);
 gboolean device_is_le(struct btd_device *device);
 gboolean device_is_temporary(struct btd_device *device);
@@ -106,9 +106,8 @@ void btd_device_unref(struct btd_device *device);
 int device_block(struct btd_device *device, gboolean update_only);
 int device_unblock(struct btd_device *device, gboolean silent,
 							gboolean update_only);
-void btd_device_set_pnpid(struct btd_device *device, uint8_t vendor_id_src,
-			uint16_t vendor_id, uint16_t product_id,
-			uint16_t product_ver);
+void btd_device_set_pnpid(struct btd_device *device, uint16_t source,
+			uint16_t vendor, uint16_t product, uint16_t version);
 GIOChannel *device_att_connect(gpointer user_data);
 
 struct btd_profile;
